
#include "Dire/MEwrap.h"

using namespace Pythia8;

bool isAvailableME(PY8MEs_namespace::PY8MEs& accessor, vector<int> in_pdgs,
    vector<int> out_pdgs, set<int> req_s_channels) {
  PY8MEs_namespace::PY8ME * query = accessor.getProcess(in_pdgs, out_pdgs,
      req_s_channels);
  return (query != 0);
}

bool isAvailableME(PY8MEs_namespace::PY8MEs& accessor,
  const Pythia8::Event& event) {

  vector <int> in_pdgs;
  in_pdgs.push_back(event[3].id());
  in_pdgs.push_back(event[4].id());
  vector <int> out_pdgs;
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() == 21 ) continue;
    if ( event[i].idAbs() < 6 ) continue;
    if ( event[i].isFinal() ) out_pdgs.push_back(event[i].id());
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() != 21 ) continue;
    if ( event[i].isFinal() ) out_pdgs.push_back(event[i].id());
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].idAbs() > 5 ) continue;
    if ( event[i].isFinal() ) out_pdgs.push_back(event[i].id());
  }

  vector<int> colors;
  colors.push_back(event[3].col()); colors.push_back(event[3].acol());
  colors.push_back(event[4].col()); colors.push_back(event[4].acol());
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() == 21 ) continue;
    if ( event[i].idAbs() < 6 ) continue;
    if ( event[i].isFinal() ) {
      colors.push_back(event[i].col());
      colors.push_back(event[i].acol());
    }
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() != 21 ) continue;
    if ( event[i].isFinal() ) {
      colors.push_back(event[i].col());
      colors.push_back(event[i].acol());
    }
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].idAbs() > 5 ) continue;
    if ( event[i].isFinal() ) {
      colors.push_back(event[i].col());
      colors.push_back(event[i].acol());
    }
  }

  set<int> req_s_channels; 

  PY8MEs_namespace::PY8ME * query = accessor.getProcess(in_pdgs, out_pdgs,
      req_s_channels);
  return (query != 0);
}

// Evaluate a given process with an accessor
double calcME(PY8MEs_namespace::PY8MEs& accessor, const Pythia8::Event& event) {

  vector <int> in_pdgs;
  in_pdgs.push_back(event[3].id());
  in_pdgs.push_back(event[4].id());
  vector <int> out_pdgs;
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() == 21 ) continue;
    if ( event[i].idAbs() < 6 ) continue;
    if ( event[i].isFinal() ) out_pdgs.push_back(event[i].id());
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() != 21 ) continue;
    if ( event[i].isFinal() ) out_pdgs.push_back(event[i].id());
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].idAbs() > 5 ) continue;
    if ( event[i].isFinal() ) out_pdgs.push_back(event[i].id());
  }

  vector<Pythia8::Vec4> p;
  p.push_back(event[3].p());
  p.push_back(event[4].p());
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() == 21 ) continue;
    if ( event[i].idAbs() < 6 ) continue;
    if ( event[i].isFinal() ) p.push_back(event[i].p());
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() != 21 ) continue;
    if ( event[i].isFinal() ) p.push_back(event[i].p());
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].idAbs() > 5 ) continue;
    if ( event[i].isFinal() ) p.push_back(event[i].p());
  }

  vector<int> colors;
  colors.push_back(event[3].col()); colors.push_back(event[3].acol());
  colors.push_back(event[4].col()); colors.push_back(event[4].acol());
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() == 21 ) continue;
    if ( event[i].idAbs() < 6 ) continue;
    if ( event[i].isFinal() ) {
      colors.push_back(event[i].col());
      colors.push_back(event[i].acol());
    }
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].id() != 21 ) continue;
    if ( event[i].isFinal() ) {
      colors.push_back(event[i].col());
      colors.push_back(event[i].acol());
    }
  }
  for (int i = 4; i < event.size(); ++i) {
    if ( event[i].idAbs() > 5 ) continue;
    if ( event[i].isFinal() ) {
      colors.push_back(event[i].col());
      colors.push_back(event[i].acol());
    }
  }

  set<int> req_s_channels; 
  accessor.getProcess(in_pdgs, out_pdgs, req_s_channels);

  vector < vec_double > p_vec = vector < vec_double > (in_pdgs.size() +
      out_pdgs.size(), vec_double(4, 0.));
  for (int i = 0; i < int(in_pdgs.size() + out_pdgs.size()); i++ ) {
    p_vec[i][0] = abs(p[i].e()) > 1e-10 ? p[i].e() : 0.0;
    p_vec[i][1] = abs(p[i].px()) > 1e-10 ? p[i].px() : 0.0;
    p_vec[i][2] = abs(p[i].py()) > 1e-10 ? p[i].py() : 0.0;
    p_vec[i][3] = abs(p[i].pz()) > 1e-10 ? p[i].pz() : 0.0;
  }

  vector<int> helicities(0); 
  pair < double, bool > res = accessor.calculateME(in_pdgs, out_pdgs, p_vec,
      req_s_channels, colors, helicities);

  if (res.second) return res.first;

  // Done
  return 0.0;

}
